
# DO NOT EDIT THIS FILE -- AUTOGENERATED BY PANTS
# Target: packages/roboto:roboto-py-sdk-dist

from setuptools import setup

setup(**{
    'entry_points': {
        'console_scripts': [
            'roboto = roboto.cli:entry',
        ],
    },
    'install_requires': (
        'boto3~=1.26',
        'botocore~=1.29',
        'cookiecutter~=2.5.0',
        'filelock~=3.15',
        'mcap-ros1-support~=0.7.1',
        'mcap-ros2-support~=0.5.3',
        'mcap~=1.1.1',
        'packaging~=24.0',
        'pathspec~=0.11',
        'pydantic-core~=2.14',
        'pydantic-settings~=2.1',
        'pydantic~=2.5',
        'tenacity~=8.2',
        'tqdm~=4.65',
        'typing-extensions~=4.8; python_version < "3.10"',
    ),
    'name': 'roboto',
    'namespace_packages': (
    ),
    'package_data': {
        'roboto': (
            'py.typed',
        ),
    },
    'packages': (
        'roboto',
        'roboto.action_runtime',
        'roboto.auth',
        'roboto.cli',
        'roboto.cli.actions',
        'roboto.cli.collections',
        'roboto.cli.command',
        'roboto.cli.common_args',
        'roboto.cli.datasets',
        'roboto.cli.extension',
        'roboto.cli.images',
        'roboto.cli.invocations',
        'roboto.cli.orgs',
        'roboto.cli.tokens',
        'roboto.cli.triggers',
        'roboto.cli.users',
        'roboto.domain',
        'roboto.domain.actions',
        'roboto.domain.collections',
        'roboto.domain.comments',
        'roboto.domain.datasets',
        'roboto.domain.devices',
        'roboto.domain.events',
        'roboto.domain.files',
        'roboto.domain.orgs',
        'roboto.domain.tokens',
        'roboto.domain.topics',
        'roboto.domain.users',
        'roboto.exceptions',
        'roboto.http',
        'roboto.image_registry',
        'roboto.notifications',
        'roboto.pydantic',
        'roboto.query',
        'roboto.upload_agent',
    ),
    'python_requires': '>=3.9,<4',
    'version': 'v0.8.1',
})
